/*
 * Common display-related functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __DISPLAY_H__
#define __DISPLAY_H__

#include <stdint.h>



/* Display configuration type. */

typedef struct
{
    /* Framebuffer pointer and size. */

    uint8_t *framebuffer;
    uint32_t screen_size;
    uint32_t line_length, line_count;   /* width, height */

    /* Screen start/top and limit pointers. */

    uint8_t *screen_start, *screen_limit;

    /* Number of scanlines per display line. */

    int line_multiplier;

    /* Number of consecutive pixels provided by a framebuffer region. */

    uint32_t cell_size;

    /* DMA transfer sizes. */

    uint32_t transfer_cell_size;

    /* Display line positions. */

    uint32_t hfreq_limit, hsync_start, hsync_end;

    /* Display region scanline positions. */

    uint32_t visible_start, vfp_start, vsync_start, vsync_end;

} display_config_t;



/* Access functions. */

int get_position(display_config_t *cfg, int x);

void test_linedata(display_config_t *cfg);

void copy_display(display_config_t *cfg, uint8_t *store, int width, int height,
                  int x, int y, int key, int to_display);

void copy_display_section(display_config_t *cfg, uint8_t *store,
                  int width, int height,
                  int xstart, int ystart, int xsize, int ysize,
                  int x, int y, int key, int to_display);

void scroll_display(display_config_t *cfg, int x, int y);

#endif /* __DISPLAY_H__ */
